(******************************************************************************)
(*  ocsigen-bundler: create self-contained Ocsigen application package        *)
(*                                                                            *)
(*  Copyright (C) 2011, OCamlCore SARL                                        *)
(*                                                                            *)
(*  This library/application is free software; you can redistribute it        *)
(*  and/or modify it under the terms of the GNU Lesser General Public         *)
(*  License as published by the Free Software Foundation; either version 2.1  *)
(*  of the License, or (at your option) any later version, with the OCaml     *)
(*  static compilation exception.                                             *)
(*                                                                            *)
(*  This library/application is distributed in the hope that it will be       *)
(*  useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file         *)
(*  COPYING for more details.                                                 *)
(*                                                                            *)
(*  You should have received a copy of the GNU Lesser General Public License  *)
(*  along with this library; if not, write to the Free Software Foundation,   *)
(*  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA             *)
(******************************************************************************)

let () = 
  let tgt = 
    ref None
  in
  let conf = 
    ref None
  in
  let dry_run = 
    ref false
  in
  let port = 
    ref None
  in
  let logdir = 
    ref None
  in
  let verbose = 
    ref false
  in
  let set_opt_string r = 
    Arg.String (fun s -> r := Some s)
  in
  let mkdir = 
    ref []
  in
  let natives =
    ref []
  in
  let copy_dir = 
    ref []
  in
  let copy_file =
    ref []
  in
  let do_bundle =
    ref true
  in
  let deploy_host = 
    ref None
  in
  let deploy_dir =
    ref None
  in
  let push_tuple r = 
    let fst_arg = ref "" in
    let snd_arg = ref "" in
      [
        Arg.Set_string fst_arg;
        Arg.Set_string snd_arg;
        Arg.Unit (fun () -> r := (!fst_arg, !snd_arg) :: !r);
      ]
  in

    Arg.parse 
      [
        "--conf",
        set_opt_string conf,
        "fn Ocsigen configuration file.";

        "--target",
        set_opt_string tgt,
        "dn Target directory.";

        "--dry-run",
        Arg.Set dry_run,
        " Don't write anything.";

        "--verbose",
        Arg.Set verbose,
        " Be verbose.";

        "--port",
        Arg.String (fun s -> port := Some (int_of_string s)),
        "n Port number to use.";

        "--logdir",
        set_opt_string logdir,
        "dn Log directory.";

        "--mkdir",
        Arg.String (fun s -> mkdir := s :: !mkdir),
        "dn Directory to create inside the target.";

        "--copy-file",
        Arg.Tuple (push_tuple copy_file),
        "fn+dn Copy a file 'fn' to a directory 'dn' inside the target.";

        "--copy-dir",
        Arg.Tuple (push_tuple copy_dir),
        "dn1+dn2 Copy a directory 'dn1' to a directory 'dn2' inside the target.";

        "--extra-native",
        Arg.String (fun s -> natives := s :: !natives),
        "cmd Copy an exec inside the target.";

        "--no-bundle",
        Arg.Clear do_bundle,
        " Don't build bundle at all, useful to just deploy target to a host.";

        "--deploy-host",
        set_opt_string deploy_host,
        "host Target host for deployment.";

        "--deploy-dir",
        set_opt_string deploy_dir,
        "dn Target directory on host for deployment.";
      ]
      (fun str ->
         failwith 
           (Printf.sprintf 
              "Don't know what to do with option '%s'"
              str))
      "ocsigen-bundler, create standalone ocsigen website, by Sylvain Le Gall\n\
       \n\
       ocsigen-bundler [options]*\n\
       \n\
       Options:\n";

    let conf, tgt = 
      match !conf, !tgt with 
        | Some fn1, Some fn2 ->
            fn1, fn2
        | None, Some _ ->
            failwith "Ocsigen configuration file is not set"
        | None, None ->
            failwith "Target directory and Ocsigen configuration are not set"
        | Some _, None ->
            failwith "Target directory is not set"
    in
      begin
        if !do_bundle then
          Ocsigen_bundler.make 
            ~verbose:!verbose
            ~dry_run:!dry_run 
            ?port:!port 
            ?logdir:!logdir
            ~mkdir:!mkdir
            ~copy_file:!copy_file
            ~copy_dir:!copy_dir
            ~natives:!natives
            conf tgt
      end;

      begin
        match !deploy_dir with 
          | Some dn ->
              Ocsigen_bundler.deploy
                ~verbose:!verbose
                ~dry_run:!dry_run
                ?host:!deploy_host
                ~tgt:dn
                tgt
          | None ->
              ()
      end

